import re

from util import *

BAD_TOKEN = r'EXIT-SUCCESS'
GOOD_TOKEN = r'Inferior 1 \(process \d+\) exited normally'

def observe_child_crash_and_exit():
    expect_gdb('Program received signal SIGSEGV')

    send_gdb('c')
    for line in iterlines_both():
        m = re.search(BAD_TOKEN, line)
        if m:
            failed('Saw illegal token "'+ BAD_TOKEN +'"')
        m = re.search(GOOD_TOKEN, line)
        if m:
            return

send_gdb('c')
observe_child_crash_and_exit()

restart_replay()
observe_child_crash_and_exit()

ok()



from util import *

send_gdb('break vsyscall_reverse_next.c:6')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('disable 1')

send_gdb('n')
expect_gdb('atomic_puts')

send_gdb('reverse-next')
expect_gdb('time')

ok()



from util import *
import re

send_gdb('b string_store')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('p buf')
expect_gdb(re.compile(r'= ([^ ]+)'))
buf = eval(last_match().group(1));

send_gdb('watch -l *(uint16_t*)%d'%(buf + 13))
expect_gdb('atchpoint 2')

send_gdb('watch -l *(char*)-1')
expect_gdb('atchpoint 3')

send_gdb('c')
expect_gdb('atchpoint 2')

send_gdb('p buf[13]')
expect_gdb('= 97')
send_gdb('p buf[14]')
expect_gdb('= 0')

ok()



from util import *
import re

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('p ((int*)&__elision_aconf)[2]')
expect_gdb(re.compile(r'= 0|No symbol'))
send_gdb('p/x *(char*)elision_init')
expect_gdb(re.compile(r'= 0xc3|No symbol'))

ok()



from util import *

send_gdb('b first_breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('b second_breakpoint')
expect_gdb('Breakpoint 2')

send_gdb('c')
# If we hit first_breakpoint, then we never continue and never reach
# second_breakpoint.
expect_gdb('Breakpoint 2, second_breakpoint')

restart_replay(1)
expect_gdb('Breakpoint 1, first_breakpoint')

ok()



import pexpect, re, signal, sys, time

__all__ = [ 'expect_gdb', 'send_gdb','expect_rr', 'expect_list',
            'restart_replay', 'interrupt_gdb', 'ok',
            'failed', 'iterlines_both', 'last_match', 'get_exe_arch' ]

# Public API
def expect_gdb(what):
    expect(gdb_rr, what)

def expect_list(pats):
    return gdb_rr.expect_list(pats)

def expect_rr(what):
    expect(gdb_rr, what)

def failed(why, e=None):
    print('FAILED:', why)
    if e:
        print('exception:', e)
    clean_up()
    sys.exit(1)

def interrupt_gdb():
    try:
        gdb_rr.kill(signal.SIGINT)
    except Exception as e:
        failed('interrupting gdb', e)
    expect_gdb('stopped.')

def iterlines_both():
    return gdb_rr

def last_match():
    return gdb_rr.match

# Restarts and continues execution
def restart_replay(event=0):
    if event:
        send_gdb('r %d'%(event))
    else:
        send_gdb('r')
    # gdb may not prompt here. It's ok to send an unnecessary 'y'
    # since there is no such command.
    send_gdb('y')
    # Wait to see 'stopped'. We don't want this to get buffered up
    # so callers expecting a 'stopped' *after* replay has resumed
    # get confused.
    expect_rr('stopped')
    send_gdb('c')

def send_gdb(what):
    send(gdb_rr, "%s\n"%what)

def ok():
    send_gdb('q')
    send_gdb('y')
    clean_up()

# Internal helpers
# Don't use python timeout. Use test-monitor timeout instead.
TIMEOUT_SEC = 10000
# gdb and rr are part of the same process tree, so they share
# stdin/stdout.
gdb_rr = None

def clean_up():
    global gdb_rr
    iterations = 0
    while gdb_rr:
        try:
            # FIXME: without this sleep python freezes instead of exiting.
            # The sleep has to be before BufferedRWPair.close()
            time.sleep(0.1)
            gdb_rr.close(force=1)
            gdb_rr = None
        except Exception as e:
            if iterations < 5:
                print("close() failed with '%s', retrying..."%e)
                iterations = iterations + 1
            else:
                gdb_rr = None

def expect(prog, what):
    try:
        prog.expect(what)
    except Exception as e:
        failed('expecting "%s"'% (what), e)

def get_exe_arch():
    send_gdb('show architecture')
    expect_gdb('The target architecture is set automatically \\(currently ([0-9a-z:-]+)\\)')
    global gdb_rr
    return gdb_rr.match.group(1)

def get_rr_cmd():
    '''Return the command that should be used to invoke rr, as the tuple
  (executable, array-of-args)'''
    rrargs = sys.argv[1:]
    return (rrargs[0], rrargs[1:])

def send(prog, what):
    try:
        prog.send(what)
    except Exception as e:
        failed('sending "%s"'% (what), e)

def set_up():
    global gdb_rr
    try:
        gdb_rr = pexpect.spawn(*get_rr_cmd(), timeout=TIMEOUT_SEC, encoding='utf-8', logfile=open('gdb_rr.log', 'w'))
        gdb_rr.delaybeforesend = 0
        expect_gdb('\(rr\)')
    except Exception as e:
        failed('initializing rr and gdb', e)

set_up()



from util import *

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('checkpoint')
expect_gdb('Checkpoint 1 at')

send_gdb('n')
send_gdb('restart 1')
expect_gdb('breakpoint')

send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('xited normally')



#!/usr/bin/env python3

import argparse
import os
import struct
import sys

parser = argparse.ArgumentParser()
group = parser.add_mutually_exclusive_group()
group.add_argument('--reset', action='store_true')
group.add_argument('--check', action='store_true')
args = parser.parse_args()

# MSRC001_1020: Load-Store Configuration
MSR = 0xc0011020
# Disable SpecLockMap
BIT = 1 << 54

if not os.path.exists('/dev/cpu/0/msr'):
    ret = os.system('modprobe msr')
    if ret:
        sys.exit(ret)

def read_msr(cpu):
    msr = os.open('/dev/cpu/{}/msr'.format(cpu), os.O_RDONLY)
    os.lseek(msr, MSR, os.SEEK_SET)
    (val,) = struct.unpack('<q', os.read(msr, 8))
    os.close(msr)
    return val

cpus = [cpu for cpu in os.listdir('/dev/cpu') if cpu.isdigit()]

if not args.check:
    for cpu in cpus:
        val = read_msr(cpu)
        if args.reset:
            if val & BIT == 0:
                continue
            val &= ~BIT
        else:
            if val & BIT:
                continue
            val |= BIT
        msr = os.open('/dev/cpu/{}/msr'.format(cpu), os.O_WRONLY)
        os.lseek(msr, MSR, os.SEEK_SET)
        os.write(msr, struct.pack('<q', val))
        os.close(msr)

ssb_status = 'unknown'
if not args.reset:
    import ctypes
    lib = ctypes.CDLL(None)
    prctl = lib.prctl
    prctl.restype = ctypes.c_int
    prctl.argtypes = (ctypes.c_int, ctypes.c_ulong, ctypes.c_ulong, ctypes.c_ulong, ctypes.c_ulong)
    PR_GET_SPECULATION_CTRL = 52
    PR_SET_SPECULATION_CTRL = 53
    PR_SPEC_STORE_BYPASS = 0
    PR_SPEC_PRCTL = 1 << 0
    PR_SPEC_DISABLE = 1 << 2
    # When the kernel does per-process SSB mitigation via prctl or seccomp, it touches the same
    # MSR that we changed, but does so based on a value of the MSR it got at boot time, so it
    # effectively will reset the bit we just set if it wasn't already set at boot time.
    # This doesn't happen when the SSB mitigation is either entirely on or off.
    # This is specific to Zen and Zen+, because Zen 2 doesn't require the kernel to change the MSR.
    # Check whether the kernel does per-process SSB mitigation, and if it does, enable it for this
    # process.
    ssb_mode = prctl(PR_GET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, 0, 0, 0)
    if ssb_mode >= 0 and ssb_mode & PR_SPEC_PRCTL:
        mitigated = (prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, PR_SPEC_DISABLE, 0, 0) == 0)
        if not mitigated:
            print('Failed to enable SSB mitigation')
        else:
            ssb_status = 'mitigated'
    else:
        ssb_status = 'immutable'


msrs = [read_msr(cpu) & BIT for cpu in cpus]

if all(msr for msr in msrs):
    if ssb_status in ('mitigated', 'immutable') or args.check:
        print('Zen workaround in place')
    else:
        print('Zen workaround maybe in place.')
elif args.reset or args.check:
    if all(not msr for msr in msrs):
        print('Zen workaround disabled')
    elif args.reset:
        print('Zen workaround somehow not entirely disabled?')
    else:
        print('Zen workaround not entirely enabled?')
else:
    print('Zen workaround does not stick. Please see https://github.com/mozilla/rr/wiki/Zen')



from util import *

send_gdb('b sighandler')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Program received signal SIGILL')

send_gdb('checkpoint')
expect_gdb('Checkpoint 1 at')

send_gdb('c')
expect_gdb('Breakpoint 1, sighandler')

send_gdb("restart 1");
send_gdb('c')
expect_gdb('Breakpoint 1, sighandler')

ok()



from util import *

send_gdb('b C')
expect_gdb('Breakpoint 1')

send_gdb('c')

expect_rr('calling C')

expect_gdb('Breakpoint 1, C')

send_gdb('bt')
expect_gdb('#0[^C]+C[^#]+#1[^B]+B[^#]+#2[^A]+A[^#]+#3[^m]+main')

ok()



from util import *
import re

send_gdb('break *cpuid_instruction_label')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
expect_gdb('(rr)')

send_gdb('p $pc')
expect_gdb(re.compile(r'(0x[a-f0-9]+)'))
pc = eval(last_match().group(1));
expect_gdb('(rr)')
send_gdb('stepi')
expect_gdb('(rr)')

send_gdb('p $pc')
expect_gdb(re.compile(r'(0x[a-f0-9]+)'))
pc2 = eval(last_match().group(1));
if pc2 - pc != 2:
    failed('Expected 0x%x after singlestep, got 0x%x'%(pc + 2, pc2))

send_gdb('c')
expect_gdb('EXIT-SUCCESS')

ok()



from io import StringIO
import sys

class RawBytes(object):
    """A sequence of literal bytes to appear in an assembly language template."""
    def __init__(self, *bytes):
        self.bytes = bytes

    def __len__(self):
        return len(self.bytes)

class Field(object):
    """A variable field of bytes."""
    def __init__(self, name, byte_length):
        self.name = name
        self.byte_length = byte_length

    def __len__(self):
        return self.byte_length

    def c_type(self):
        types = { 8: 'uint64_t', 4: 'uint32_t', 2: 'uint16_t', 1: 'uint8_t' }
        return types[self.byte_length]

class ShiftField(object):
    """A field embedded at some bit shift offset in another object."""
    def __init__(self, parent, shift, name, byte_length):
        self.parent = parent
        self.shift = shift
        self.name = name
        self.byte_length = byte_length

    def __len__(self):
        return len(self.parent)

    def c_type(self):
        types = { 8: 'uint64_t', 4: 'uint32_t', 2: 'uint16_t', 1: 'uint8_t' }
        return types[self.byte_length]

    def patch_c_type(self):
        types = { 8: 'uint64_t', 4: 'uint32_t', 2: 'uint16_t', 1: 'uint8_t' }
        return types[len(self.parent)]

class AssemblyTemplate(object):
    """A sequence of RawBytes and Field objects, which can be used to verify
    that a given sequence of assembly instructions matches the RawBytes while
    pulling out the Field values for inspection.  Or for creating custom
    assembly stubs, filling out Fields with runtime-determined values."""
    def __init__(self, *chunks):
        # Merge consecutive RawBytes elements together for efficiency of
        # matching and for simplicity of template expansion.
        merged_chunks = []
        current_raw_bytes = []
        for c in chunks:
            if isinstance(c, Field) or isinstance(c, ShiftField):
                # Push any raw bytes before this.
                if current_raw_bytes:
                    merged_chunks.append(RawBytes(*current_raw_bytes))
                    current_raw_bytes = []
                merged_chunks.append(c)
            else:
                current_raw_bytes.extend(c.bytes)
        # Merge in trailing raw bytes.
        if current_raw_bytes:
            merged_chunks.append(RawBytes(*current_raw_bytes))
        self.chunks = merged_chunks

    def fields(self):
        return [c for c in self.chunks if (isinstance(c, Field) or isinstance(c, ShiftField))]

    def bytes(self):
        bytes = []
        for c in self.chunks:
            if isinstance(c, Field):
                bytes.extend([0] * len(c))
            elif isinstance(c, ShiftField):
                bytes.extend(c.parent.bytes)
            else:
                bytes.extend(c.bytes)
        return bytes

templates = {
    'X86SysenterVsyscallImplementation': AssemblyTemplate(
        RawBytes(0x51),         # push %ecx
        RawBytes(0x52),         # push %edx
        RawBytes(0x55),         # push %ebp
        RawBytes(0x89, 0xe5),   # mov %esp,%ebp
        RawBytes(0x0f, 0x34),   # sysenter
    ),
    'X86SysenterVsyscallImplementationAMD': AssemblyTemplate(
        RawBytes(0x51),         # push %ecx
        RawBytes(0x52),         # push %edx
        RawBytes(0x55),         # push %ebp
        RawBytes(0x89, 0xcd),   # mov %ecx,%ebp
        RawBytes(0x0f, 0x05),   # syscall
        RawBytes(0xcd, 0x80),   # int $0x80
    ),
    'X86SysenterVsyscallUseInt80': AssemblyTemplate(
        RawBytes(0xcd, 0x80),   # int $0x80
        RawBytes(0xc3),         # ret
    ),
    'X86SysenterVsyscallSyscallHook': AssemblyTemplate(
        RawBytes(0xe9),         # jmp $syscall_hook_trampoline
        Field('syscall_hook_trampoline', 4),
    ),
    'X86VsyscallMonkeypatch': AssemblyTemplate(
        RawBytes(0xb8),         # mov $syscall_number,%eax
        Field('syscall_number', 4),
        RawBytes(0xe8),         # call $X86VsyscallMonkeypatchShared
        Field('vsyscall_monkeypatch_shared', 4),
        RawBytes(0xc3),
    ),
    'X86VsyscallMonkeypatchShared': AssemblyTemplate(
        # __vdso functions use the C calling convention, so
        # we have to set up the syscall parameters here.
        # No x86-32 __vdso functions take more than two parameters.
        RawBytes(0x53),         # push %ebx
        RawBytes(0x8b, 0x5c, 0x24, 0x0c), # mov 12(%esp),%ebx
        RawBytes(0x8b, 0x4c, 0x24, 0x10), # mov 16(%esp),%ecx
        RawBytes(0xcd, 0x80),   # int $0x80
        # pad with NOPs to make room to dynamically patch the syscall
        # with a call to the preload library, once syscall buffering
        # has been initialized.
        RawBytes(0x90),         # nop
        RawBytes(0x90),         # nop
        RawBytes(0x90),         # nop
        RawBytes(0x5b),         # pop %ebx
        RawBytes(0xc3),         # ret
    ),
    'X86SyscallStubExtendedJump': AssemblyTemplate(
        # This code must match the stubs in syscall_hook.S.
        RawBytes(0x89, 0x25, 0x08, 0x10, 0x00, 0x70), # movl %esp,(stub_scratch_1)
        RawBytes(0xFF, 0x05, 0x0c, 0x10, 0x00, 0x70), # incl (alt_stack_nesting_level)
        RawBytes(0x83, 0x3c, 0x25, 0x0c, 0x10, 0x00, 0x70, 0x01), # cmpl 1,(alt_stack_nesting_level)
        RawBytes(0x75, 0x06),                                     # jne dont_switch
        RawBytes(0x8b, 0x25, 0x00, 0x10, 0x00, 0x70), # movl (syscallbuf_stub_alt_stack),%esp
        # dont_switch:
        RawBytes(0xff, 0x35, 0x08, 0x10, 0x00, 0x70), # pushl (stub_scratch_1)
        RawBytes(0x68),                               # pushl $return_addr
        Field('return_addr', 4),
        RawBytes(0xe9),                               # jmp $trampoline_relative_addr
        Field('trampoline_relative_addr', 4)
    ),

    'X64JumpMonkeypatch': AssemblyTemplate(
        RawBytes(0xe9),         # jmp $relative_addr
        Field('relative_addr', 4),
    ),
    'X64VsyscallMonkeypatch': AssemblyTemplate(
        RawBytes(0xb8),         # mov $syscall_number,%eax
        Field('syscall_number', 4),
        RawBytes(0x0f, 0x05),   # syscall
        # pad with NOPs to make room to dynamically patch the syscall
        # with a call to the preload library, once syscall buffering
        # has been initialized.
        RawBytes(0x90),         # nop
        RawBytes(0x90),         # nop
        RawBytes(0x90),         # nop
        RawBytes(0xc3),         # ret
    ),
    'X64SyscallStubExtendedJump': AssemblyTemplate(
        # This code must match the stubs in syscall_hook.S.
        RawBytes(0x48, 0x89, 0x24, 0x25, 0x10, 0x10, 0x00, 0x70), # movq %rsp,(stub_scratch_1)
        RawBytes(0xFF, 0x04, 0x25, 0x18, 0x10, 0x00, 0x70),       # incl (alt_stack_nesting_level)
        RawBytes(0x83, 0x3c, 0x25, 0x18, 0x10, 0x00, 0x70, 0x01), # cmpl 1,(alt_stack_nesting_level)
        RawBytes(0x75, 0x0a),                                     # jne dont_switch
        RawBytes(0x48, 0x8b, 0x24, 0x25, 0x00, 0x10, 0x00, 0x70), # movq (syscallbuf_stub_alt_stack),%rsp
        RawBytes(0xeb, 0x07),                                     # jmp after_adjust
        # dont_switch:
        RawBytes(0x48, 0x81, 0xec, 0x00, 0x01, 0x00, 0x00), # subq $256, %rsp
        # after adjust
        RawBytes(0xff, 0x34, 0x25, 0x10, 0x10, 0x00, 0x70), # pushq (stub_scratch_1)
        RawBytes(0x50),                                     # pushq rax
        RawBytes(0xc7, 0x04, 0x24),                         # movl $return_addr_lo,(%rsp)
        Field('return_addr_lo', 4),
        RawBytes(0xc7, 0x44, 0x24, 0x04),                   # movl $return_addr_hi,(%rsp+4)
        Field('return_addr_hi', 4),
        RawBytes(0xff, 0x25, 0x00, 0x00, 0x00, 0x00),       # jmp *0(%rip)
        Field('jump_target', 8),
    ),
    'X64DLRuntimeResolve': AssemblyTemplate(
        RawBytes(0x53),                   # push %rbx
        RawBytes(0x48, 0x89, 0xe3),       # mov %rsp,%rbx
        RawBytes(0x48, 0x83, 0xe4, 0xf0), # and $0xfffffffffffffff0,%rsp
    ),
    'X64DLRuntimeResolve2': AssemblyTemplate(
        RawBytes(0x53),                   # push %rbx
        RawBytes(0x48, 0x89, 0xe3),       # mov %rsp,%rbx
        RawBytes(0x48, 0x83, 0xe4, 0xc0), # and $0xffffffffffffffc0,%rsp
    ),
    'X64DLRuntimeResolvePrelude': AssemblyTemplate(
        RawBytes(0xd9, 0x74, 0x24, 0xe0),                               # fstenv -32(%rsp)
        RawBytes(0x48, 0xc7, 0x44, 0x24, 0xf4, 0x00, 0x00, 0x00, 0x00), # movq $0,-12(%rsp)
        RawBytes(0xd9, 0x64, 0x24, 0xe0),                               # fldenv -32(%rsp)
        RawBytes(0x53),                   # push %rbx
        RawBytes(0x48, 0x89, 0xe3),       # mov %rsp,%rbx
        RawBytes(0x48, 0x83, 0xe4, 0xc0), # and $0xffffffffffffffc0,%rsp
        RawBytes(0xe9),                   # jmp $relative_addr
        Field('relative_addr', 4),
    ),
    'X64EndBr': AssemblyTemplate(
        RawBytes(0xf3, 0x0f, 0x1e, 0xfa)
    ),
    'X86EndBr': AssemblyTemplate(
        RawBytes(0xf3, 0x0f, 0x1e, 0xfb)
    ),
    'X64VSyscallEntry': AssemblyTemplate(
        RawBytes(0x48, 0xc7, 0xc0), # movq $[addr], %rax
        Field('addr', 4),
        RawBytes(0xff, 0xd0) # callq *%rax
    ),
    'X64VSyscallReplacement': AssemblyTemplate(
        RawBytes(0x48, 0xc7, 0xc0), # movq $[syscallno], %rax
        Field('syscallno', 4),
        RawBytes(0x0f, 0x05) # syscall
    ),
    'ARM64VdsoMonkeypatch': AssemblyTemplate(
        ShiftField(
            RawBytes(0x08, 0x00, 0x80, 0xd2),   # movz x8, #0
            5, 'syscall_number', 2),
        RawBytes(0x01, 0x00, 0x00, 0xd4),   # svc #0
        RawBytes(0xc0, 0x03, 0x5f, 0xd6),   # ret
    ),
}

def byte_array_name(name):
    return '%s_bytes' % name

def generate_match_method(byte_array, template):
    s = StringIO()
    fields = template.fields()
    field_types = [f.c_type() for f in fields]
    field_names = [f.name for f in fields]
    args = ', ' + ', '.join("%s* %s" % (t, n) for t, n in zip(field_types, field_names)) \
           if fields else ''

    s.write('  static bool match(const uint8_t* buffer %s) {\n' % (args,))
    offset = 0
    for chunk in template.chunks:
        if isinstance(chunk, Field):
            field_name = chunk.name
            s.write('    memcpy(%s, &buffer[%d], sizeof(*%s));\n'
                    % (field_name, offset, field_name))
        elif isinstance(chunk, ShiftField):
            s.write('    (void)%s;' % chunk.name)
            s.write('    assert(0 && "Matching not implemented for ShiftField");')
        else:
            s.write('    if (memcmp(&buffer[%d], &%s[%d], %d) != 0) { return false; }\n'
                    % (offset, byte_array, offset, len(chunk)))
        offset += len(chunk)
    s.write('    return true;\n')
    s.write('  }')
    return s.getvalue()

def generate_substitute_chunk(s, chunk, byte_array, offset):
    if isinstance(chunk, Field):
        field_name = chunk.name
        s.write('    memcpy(&buffer[%d], &%s, sizeof(%s));\n'
                % (offset, field_name, field_name))
    elif isinstance(chunk, ShiftField):
        generate_substitute_chunk(s, chunk.parent, byte_array, offset);
        typ = chunk.patch_c_type()
        field_name = chunk.name
        s.write('    *((%s*)&buffer[%d]) |= (((%s)%s)<<%d);\n'
                % (typ, offset, typ, field_name, chunk.shift))
    else:
        s.write('    memcpy(&buffer[%d], &%s[%d], %d);\n'
                % (offset, byte_array, offset, len(chunk)))
    offset += len(chunk)
    return offset

def generate_substitute_method(byte_array, template):
    s = StringIO()
    fields = template.fields()
    field_types = [f.c_type() for f in fields]
    field_names = [f.name for f in fields]
    args = ', ' + ', '.join("%s %s" % (t, n) for t, n in zip(field_types, field_names)) \
           if fields else ''

    s.write('  static void substitute(uint8_t* buffer %s) {\n' % (args,))
    offset = 0
    for chunk in template.chunks:
        offset = generate_substitute_chunk(s, chunk, byte_array, offset)
    s.write('  }')
    return s.getvalue()

def generate_field_end_methods(byte_array, template):
    s = StringIO()
    offset = 0
    for chunk in template.chunks:
        offset += len(chunk)
        if isinstance(chunk, Field):
            s.write('  static const size_t %s_end = %d;\n' % (chunk.name, offset))
    return s.getvalue()

def generate_size_member(byte_array):
    s = StringIO()
    s.write('  static const size_t size = sizeof(%s);' % byte_array)
    return s.getvalue()

def generate(f):
    # Raw bytes.
    for name, template in templates.items():
        bytes = template.bytes()
        f.write('static const uint8_t %s[] = { %s };\n'
                % (byte_array_name(name), ', '.join(['0x%x' % b for b in bytes])))
    f.write('\n')

    # Objects representing assembly templates.
    for name, template in templates.items():
        byte_array = byte_array_name(name)
        f.write("""class %(class_name)s {
public:
%(match_method)s

%(substitute_method)s

%(field_end_methods)s
%(size_member)s
};
""" % { 'class_name': name,
        'match_method': generate_match_method(byte_array, template),
        'substitute_method': generate_substitute_method(byte_array, template),
        'field_end_methods': generate_field_end_methods(byte_array, template),
        'size_member': generate_size_member(byte_array), })
        f.write('\n\n')



from util import *

send_gdb('watch -l x')
expect_gdb('Hardware[()/a-z ]+watchpoint 1')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 1')

send_gdb('c')
expect_gdb('Old value = 1')
expect_gdb('New value = -1931448864')

send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited normally')

ok()



from util import *

# Signal all processes in the process group
send_gdb('!kill -WINCH 0')
send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited normally')

send_gdb('break main')
expect_gdb('Breakpoint 1')
send_gdb('run')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('!kill -WINCH 0')
send_gdb('reverse-cont')
expect_gdb('stopped')

ok()



import re
from util import *

send_gdb('b rdtsc')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('disass')
expect_gdb(re.compile(r'=> ([0-9a-fx]+) <\+[0-9]+>:\trdtsc'))

addr = last_match().group(1)

send_gdb('stepi')
send_gdb('disass')
expect_gdb(re.compile(r'=> ([0-9a-fx]+) '))

addr2 = last_match().group(1)

if eval(addr) + 2 != eval(addr2):
  failed("stepi from rdtsc at %s ended at incorrect %s" % (addr, addr2));

ok()



from util import *

send_gdb('break string_store')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('disable')
send_gdb('watch -l p[0]')
expect_gdb('watchpoint')
send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 1')
send_gdb('p p[0]')
expect_gdb('= 1')
send_gdb('p p[1]')
expect_gdb('= 0')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('break breakpoint2')
expect_gdb('Breakpoint 2')
send_gdb('reverse-cont')
expect_gdb('Breakpoint 2')

ok()



from util import *
import re

send_gdb('reverse-cont')
expect_gdb('stopped')
send_gdb('reverse-cont')
expect_gdb('stopped')
send_gdb('reverse-stepi')
expect_gdb('stopped')
send_gdb('reverse-cont')
expect_gdb('stopped')

send_gdb('stepi')
send_gdb('reverse-cont')
expect_gdb('stopped')
send_gdb('reverse-cont')
expect_gdb('stopped')
send_gdb('reverse-stepi')
expect_gdb('stopped')
send_gdb('reverse-cont')
expect_gdb('stopped')

send_gdb('stepi')
send_gdb('reverse-stepi')
send_gdb('reverse-stepi')
expect_gdb('stopped')
send_gdb('reverse-cont')
expect_gdb('stopped')

ok()



from util import *
import re

send_gdb('handle SIGKILL stop')

send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks = eval(last_match().group(1));
if ticks != 0:
    failed('ERROR in first "when-ticks"')

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('del 1')

send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks2 = eval(last_match().group(1));
if ticks2 < 0 or ticks2 > 1000000:
    failed('ERROR in second "when-ticks"')
if ticks2 <= ticks:
    failed('ERROR ... "when-ticks" failed to advance')

send_gdb('c')

send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks3 = eval(last_match().group(1));
if ticks3 < 0 or ticks3 > 1000000:
    failed('ERROR in second "when-ticks"')
if ticks3 <= ticks2:
    failed('ERROR ... "when-ticks" failed to advance')

send_gdb("seek-ticks %d" % ticks2)
expect_gdb("Program stopped.")
send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks4 = eval(last_match().group(1));
if ticks4 != ticks2:
    failed('ERROR: Failed to seek back to ticks2')

send_gdb("seek-ticks %d" % ticks)
expect_gdb("Program stopped.")
send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks5 = eval(last_match().group(1));
if ticks5 != ticks:
    failed('ERROR: Failed to seek back to ticks')

send_gdb("seek-ticks %d" % ticks2)
expect_gdb("Program stopped.")
send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks6 = eval(last_match().group(1));
if ticks6 != ticks2:
    failed('ERROR: Failed to seek forwards to ticks2')

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('check')
expect_gdb('Checkpoint 1 at')

send_gdb('p atomic_printf("hello%s", "kitty")')
expect_gdb('hellokitty')

restart_replay()
expect_gdb('Breakpoint 1')

send_gdb('p atomic_printf("hello%s", "kitty")')
expect_gdb('hellokitty')

send_gdb('restart 1')
expect_gdb('stopped')
send_gdb('c')
expect_rr('EXIT-SUCCESS')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('checkpoint')
expect_gdb('Checkpoint 1 at')
send_gdb('restart 1')
expect_gdb('stopped')
send_gdb('rc')
expect_gdb('stopped')
send_gdb('restart 1')
expect_gdb('stopped')
send_gdb('c')
expect_gdb('SIGKILL')

ok()



import sys
import re

if len(sys.argv) < 4:
    print('''Usage: %s <syscall-name> <perf-counter-name> <expected-perf-events-between-syscalls>
Exits with status 0 if exactly the expected number of perf events occur between
every pair of consecutive system calls of the given type.''' % sys.argv[0])
    sys.exit(2)

syscall = sys.argv[1]
counter = sys.argv[2]
expected_count = int(sys.argv[3])

last_perfctr_value = -1
syscall_re = re.compile("`SYSCALL: (\\w+)' \\(state:0\\)")
perfctr_re = re.compile(counter + ":(\\d+)")

while True:
    line = sys.stdin.readline()
    if not line:
        sys.exit(0)
    m = syscall_re.search(line)
    if m:
        if m.group(1) == syscall:
            line = sys.stdin.readline()
            m = perfctr_re.search(line)
            if m:
                v = int(m.group(1))
                if last_perfctr_value >= 0 and v - last_perfctr_value != expected_count:
                    print("Mismatch: saw %d %ss between %ss (from %d to %d), expected %d" % \
                      (v - last_perfctr_value, counter, syscall, last_perfctr_value, v, expected_count))
                    sys.exit(1)
                last_perfctr_value = v
        else:
            # Ignore nonconsecutive syscalls. In the cpuid test, we have
            # two batches of geteuid32s; one injected by rr itself to detect
            # a buggy system, and a separate one for the test. We need to
            # ignore the geteuid32 pair that spans the gap between the batches.
            last_perfctr_value = -1



from util import *
import re

arch = get_exe_arch()

if arch == 'aarch64':
    send_gdb('p $v0.d.u')
    expect_gdb('{0, 0}')
elif arch == 'i386' or arch == 'i386:x86-64':
    send_gdb('p $xmm0')
    expect_gdb('v4_float = {0, 0, 0, 0}')

ok()



from util import *

send_gdb('break string_store')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('watch -l p[0]')
expect_gdb('watchpoint 2')
send_gdb('c')
expect_gdb('watchpoint 2')

send_gdb('break')
expect_gdb('Breakpoint 3')
send_gdb('disable 3')
send_gdb('finish')
expect_gdb('main')

send_gdb('enable 3')
send_gdb('reverse-continue')
expect_gdb('Breakpoint 3')

ok()



#!/usr/bin/env python3

import sys
import multiprocessing
import tempfile
import subprocess
import copy
import shutil
import os
import itertools

objdir = sys.argv[1]
sanity_runs = eval(sys.argv[2])
runs = eval(sys.argv[3])
name = sys.argv[4]
params = sys.argv[5:]

GOOD_FAIL = 77

# Runs test, returns True if the test passed, False if it
# expectedly fails. Exits with code 1 if test unexpectedly failed.
def run(rr_params):
    d = tempfile.mkdtemp(prefix='rr-chaos-')
    try:
        env = copy.copy(os.environ)
        env['_RR_TRACE_DIR'] = d
        with open(d + "/out", 'w') as out:
            p = subprocess.Popen(["%s/bin/rr"%objdir, "record"] + rr_params + ["%s/bin/%s"%(objdir, name)] + params, env=env,
                stdout=out, stderr=out)
            ret = p.wait()
        if ret != 0 and ret != GOOD_FAIL:
            print("Test %s failed unexpectedly; leaving behind trace in %s"%(name, d))
        out_array = []
        with open(d + "/out", 'r') as out:
            for line in out:
                out_array.append(line)
        return [ret, out_array]
    finally:
        shutil.rmtree(d)

# Use only half the cores. Otherwise tests will induce starvation
# themselves; we want to measure starvation induced by rr.
pool = multiprocessing.Pool(max(1, round(multiprocessing.cpu_count()/2)))

def safe_exit(code):
    pool.terminate()
    pool.join()
    sys.exit(code)

print("Running %d iterations of %s/bin/%s %s without chaos mode"%(sanity_runs, objdir, name, ' '.join(params)))
sanity_failed = 0
for r in pool.imap_unordered(run, itertools.repeat([], sanity_runs)):
    if r[0] == 0:
        continue
    if r[0] != GOOD_FAIL:
        safe_exit(r[0])
        break
    sanity_failed = sanity_failed + 1
if sanity_failed == sanity_runs:
    print("PROBLEM: %d runs of %s all failed; not a good chaos mode test"%(sanity_failed, name))
    safe_exit(2)
print("Without chaos mode, %d runs of %s failed out of %d"%(sanity_failed, name, sanity_runs))

print("Running %d iterations of %s/bin/%s %s in chaos mode"%(runs, objdir, name, ' '.join(params)))
failed = 0
for r in pool.imap_unordered(run, itertools.repeat(["--chaos"], runs)):
    if r[0] == 0:
        continue
    if r[0] != GOOD_FAIL:
        safe_exit(r[0])
        break
    if failed == 0:
        print("First test failure detected, output:")
        for line in r[1]:
            print(line,)
    failed = failed + 1
if failed == 0:
    print("PROBLEM: With chaos mode, test %s did not fail in %d runs"%(name, runs))
    safe_exit(1)

print("With chaos mode, %d runs of %s failed out of %d"%(failed, name, runs))
if float(failed)/runs < 3*float(sanity_failed)/sanity_runs:
    print("PROBLEM: Chaos mode didn't really help!")
    safe_exit(3)

safe_exit(0)
print()



from util import *
import re

# Setup breakpoints
send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('b breakpoint2')
expect_gdb('Breakpoint 2')
send_gdb('b breakpoint3')
expect_gdb('Breakpoint 3')

# Empty checkpoint list
send_gdb('info checkpoint')
expect_gdb('No checkpoints.')

# Create checkpoint at each breakpoint
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('checkpoint')
index = expect_list([re.compile(r'Checkpoint 1 at .*'), re.compile(r'ERROR')])
if index > 0:
    failed('ERROR detected in rr output')
send_gdb('c')
expect_gdb('Breakpoint 2, breakpoint2')
send_gdb('checkpoint')
expect_gdb('Checkpoint 2 at')
send_gdb('c')
expect_gdb('Breakpoint 3, breakpoint3')
send_gdb('checkpoint')
expect_gdb('Checkpoint 3 at')

# Checkpoint list
send_gdb('info checkpoint')
expect_gdb('ID\tWhen\tWhere')
expect_gdb('1\t')
expect_gdb('2\t')
expect_gdb('3\t')

# Resume checkpoints: each one stops at its breakpoint
send_gdb("restart 1");
expect_gdb('stopped')
send_gdb('c')
expect_gdb('Breakpoint 2, breakpoint2')
send_gdb("restart 3");
expect_gdb('stopped')
send_gdb('c')
expect_rr('exited normally')
send_gdb("restart 2");
expect_gdb('stopped')
send_gdb('c')
expect_gdb('Breakpoint 3, breakpoint3')

# Bare 'run' defaults to last resumed checkpoint
restart_replay()
expect_gdb('Breakpoint 3, breakpoint3')

# Remove checkpoint 2 and try resuming it; it should fail
send_gdb('delete checkpoint 2')
send_gdb("restart 2");
send_gdb('c')
expect_gdb('failed')

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('call crash()')
expect_gdb('SIGSEGV')
restart_replay()
expect_gdb('Breakpoint 1')

ok()



from util import *

send_gdb('break main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

# Test write watchpoint

send_gdb('p &var')
expect_gdb(r'\$1 = \(volatile int \*\) ')

send_gdb('watch *$1')
expect_gdb('Hardware[()/a-z ]+watchpoint 2')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 42')

send_gdb('c')
expect_gdb('Old value = 42')
expect_gdb('New value = 1337')

restart_replay()
expect_gdb('Breakpoint 1')

# Test read-write watchpoint

send_gdb('delete 2')
send_gdb('awatch *$1')
expect_gdb('Hardware[()/a-z ]+watchpoint 3')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 42')

send_gdb('c')
expect_gdb('Old value = 42')
expect_gdb('New value = 1337')

send_gdb('c')
expect_gdb('Value = 1337')

restart_replay()
expect_gdb('Breakpoint 1')

# Test read watchpoint. x86 treats these as read-write.

send_gdb('delete 3')
send_gdb('rwatch *$1')
expect_gdb('Hardware[()/a-z ]+watchpoint 4')

send_gdb('c')
expect_gdb('Value = 42')

send_gdb('c')
expect_gdb('Value = 1337')

send_gdb('c')
expect_gdb('Value = 1337')

send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited normally')

ok()



from util import *
import re

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1, breakpoint')

# See fxregs.c for the list of constants that are loaded into the
# $st*, $xmm* and $ymm* registers.
for i in range(8):
    send_gdb('p $st%d'%(i))
    expect_gdb(' = %d'%(i + 1))

for i in range(8):
    send_gdb('p $xmm%d.v4_float[0]'%(i))
    expect_gdb(' = %d'%(i + 10))

send_gdb('show architecture')
have_64 = 0 == expect_list([re.compile('i386:x86-64\)'), re.compile('i386\)')])

if have_64:
    for i in range(8,16):
        send_gdb('p $xmm%d.v4_float[0]'%(i))
        expect_gdb(' = %d'%(i + 10))

send_gdb('p AVX_enabled')
have_AVX = 0 == expect_list([re.compile(' = 1'), re.compile(' = 0')])

if have_AVX:
    for i in range(8):
        send_gdb('p $ymm%d.v8_float[0]'%(i))
        expect_gdb(' = %d'%(i + 10))
        send_gdb('p $ymm%d.v8_float[4]'%(i))
        expect_gdb(' = %d'%((i + 1)%8 + 10))
    if have_64:
        for i in range(8,16):
            send_gdb('p $ymm%d.v8_float[0]'%(i))
            expect_gdb(' = %d'%(i + 10))
            send_gdb('p $ymm%d.v8_float[4]'%(i))
            expect_gdb(' = %d'%((i - 8 + 1)%8 + 18))

ok()



from util import *

send_gdb('b bad_breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('b good_breakpoint')
expect_gdb('Breakpoint 2')

send_gdb('c')
# If we hit bad_breakpoint, then we never continue and never reach
# good_breakpoint.
expect_gdb('Breakpoint 2, good_breakpoint')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('check')
expect_gdb('Checkpoint 1 at')

send_gdb('c')
expect_gdb('exited normally')

send_gdb('restart 1')
expect_gdb('stopped')
send_gdb('c')
expect_gdb('exited normally')

ok()



from util import *

send_gdb('break do_checkpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('checkpoint')
expect_gdb('Checkpoint 1')
send_gdb('c')
expect_gdb('xited normally')
send_gdb('restart 1')
expect_gdb('stopped')
send_gdb('c')
expect_gdb('xited normally')
send_gdb('c')

ok()



from util import *

send_gdb('c')
expect_gdb('stopped')
send_gdb('stepi')
expect_gdb('stopped')
send_gdb('stepi')
expect_gdb('stopped')
send_gdb('c')
expect_gdb('stopped')
send_gdb('b execve')
expect_gdb('Breakpoint 1')
send_gdb('rc')
expect_gdb('Breakpoint 1')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('find buf,+1,(char)0')
expect_gdb('<buf>')
expect_gdb('1 pattern found')

send_gdb('find buf,+1000,(char)99')
expect_gdb('Pattern not found')

send_gdb('find buf,+1000,(char)2')
expect_gdb('<buf')
expect_gdb('<buf')
expect_gdb('2 patterns found')

send_gdb('find buf,+14,(int)0')
expect_gdb('<buf')
expect_gdb('<buf')
expect_gdb('2 patterns found')

send_gdb('find buf,+1000,(char)1,(char)2')
expect_gdb('<buf')
expect_gdb('1 pattern found')

send_gdb('find p + (p_end - p)/2, +1, (char)0')
expect_gdb('1 pattern found')

send_gdb('find p,p_end,(char)1')
expect_gdb('2 patterns found')

send_gdb('find p, p + (p_end - p)/2,(char)0,(char)0,(char)1')
expect_gdb('Pattern not found')

# Test search string crossing page boundaries
send_gdb('find p, p_end,(char)0,(char)0,(char)1')
expect_gdb('1 pattern found')

# Search the whole address space
send_gdb('find 0,-10L,0xabcdef01')
expect_gdb('Pattern not found')

send_gdb('find 0,-10L,(char)0,(char)1,(char)2,(char)2,(char)3,(char)0xff,(char)0xfa,(char)0xde,(char)0xbc')
# One pattern in 'buf', and two in 'p'
expect_gdb('<buf>')
expect_gdb('3 patterns found')

send_gdb('up');
send_gdb('find 0,-10L,&argc')
expect_gdb('<argc_ptr>')
send_gdb('down');

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('b mprotect')
expect_gdb('Breakpoint 2')
send_gdb('c')
expect_gdb('Breakpoint 2')

# step through mprotect() until we reach the system call that
# gets performed during replay
for i in range(0,200):
    send_gdb('stepi')
    expect_gdb('(rr)')

ok()



from util import *

# XXX this test is racy, because we don't have a way to halt replay
# until some condition is satisfied.  Maybe we should add that.
send_gdb('c')
expect_rr('spinning')
interrupt_gdb()

ok()



from util import *

send_gdb('break string_store')
expect_gdb('Breakpoint 1')

# string_store 1-byte forwards
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('finish')
send_gdb('watch -l p[1000000]')
expect_gdb('watchpoint')
send_gdb('reverse-continue')
expect_gdb('Old value = 97')
expect_gdb('New value = 0')
send_gdb('p p[999999]')
expect_gdb('= 97')
send_gdb('p p[1000000]')
expect_gdb('= 0')
send_gdb('p p[1000001]')
expect_gdb('= 0')
send_gdb('disable')
send_gdb('enable 1')

# string_store 1-byte backwards
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('finish')
send_gdb('watch -l p[1000000]')
expect_gdb('watchpoint')
send_gdb('reverse-continue')
expect_gdb('Old value = 98')
expect_gdb('New value = 97')
send_gdb('p p[1000001]')
expect_gdb('= 98')
send_gdb('p p[1000000]')
expect_gdb('= 97')
send_gdb('p p[999999]')
expect_gdb('= 97')
send_gdb('disable')
send_gdb('enable 1')

# string_store 2-bytes forwards
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('finish')
send_gdb('watch -l p[1000001]')
expect_gdb('watchpoint')
send_gdb('reverse-continue')
expect_gdb('Old value = 97')
expect_gdb('New value = 0')
send_gdb('p p[999999]')
expect_gdb('= 97')
send_gdb('p p[1000000]')
expect_gdb('= 0')
send_gdb('p p[1000001]')
expect_gdb('= 0')
send_gdb('disable')
send_gdb('enable 1')

# string_store 2-bytes backwards
# Check that a watch at the end of the loop is OK
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('finish')
send_gdb('watch -l p[0]')
expect_gdb('watchpoint')
send_gdb('reverse-continue')
expect_gdb('Old value = 98')
expect_gdb('New value = 97')
send_gdb('p p[0]')
expect_gdb('= 97')
send_gdb('disable')
send_gdb('enable 1')

# string_store 4-bytes forwards
# Just check that the late-watchpoint quirk is suppressed
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('watch -l p[800000]')
expect_gdb('watchpoint')
send_gdb('continue')
expect_gdb('Old value = 0')
expect_gdb('New value = 97')
send_gdb('p p[800003]')
expect_gdb('= 97')
send_gdb('p p[800004]')
expect_gdb('= 0')

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('watch -l *(int*)$pc')
expect_gdb('Hardware watchpoint 2')

send_gdb('reverse-continue')
expect_gdb('stopped')

ok()



from util import *

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('b breakpoint_thread')
expect_gdb('Breakpoint 2')

send_gdb('set scheduler-locking off')
for i in range(50):
  send_gdb('reverse-step')

expect_gdb('Breakpoint 2')

ok()



from util import *

send_gdb('b constructor')
expect_gdb('Make breakpoint pending on future shared library load?')

send_gdb('y')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1')

ok()



from util import *
import re

send_gdb('when')
expect_gdb(re.compile(r'Current event: (\d+)'))
t = eval(last_match().group(1));
if t < 1 or t > 10000:
    failed('ERROR in first "when"')

send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks = eval(last_match().group(1));
if ticks != 0:
    failed('ERROR in first "when-ticks"')

send_gdb('when-tid')
expect_gdb(re.compile(r'Current tid: (\d+)'))
tid = eval(last_match().group(1));

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')

send_gdb('when')
expect_gdb(re.compile(r'Current event: (\d+)'))
t2 = eval(last_match().group(1));
if t2 < 1 or t2 > 10000:
    failed('ERROR in second "when"')
if t2 <= t:
    failed('ERROR ... "when" failed to advance')

send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks2 = eval(last_match().group(1));
if ticks2 < 0 or ticks2 > 1000000:
    failed('ERROR in second "when-ticks"')
if ticks2 <= ticks:
    failed('ERROR ... "when-ticks" failed to advance')

send_gdb('when-tid')
expect_gdb(re.compile(r'Current tid: (\d+)'))
tid2 = eval(last_match().group(1));
if tid2 != tid:
    failed('ERROR ... tid changed')

# Ensure 'when' terminates a diversion
send_gdb('call strlen("abcd")')
send_gdb('when')
expect_gdb(re.compile(r'Current event: (\d+)'))
t3 = eval(last_match().group(1));
if t3 != t2:
    failed('ERROR ... diversion changed event')

send_gdb('call strlen("abcd")')
send_gdb('when-ticks')
expect_gdb(re.compile(r'Current tick: (\d+)'))
ticks3 = eval(last_match().group(1));
if ticks3 != ticks2:
    failed('ERROR ... diversion changed ticks')

send_gdb('call strlen("abcd")')
send_gdb('when-tid')
expect_gdb(re.compile(r'Current tid: (\d+)'))
tid3 = eval(last_match().group(1));
if tid3 != tid2:
    failed('ERROR ... diversion changed tid')

ok()



#!/usr/bin/env python3

import assembly_templates
from io import StringIO
import os
import string
import sys
import syscalls

def arch_syscall_number(arch, syscall):
    s = getattr(syscall[1], arch)
    if s == None:
        s = -1
    return s

def write_syscall_enum(f, arch):
    f.write("enum Syscalls {\n")
    undefined_syscall = -1
    for name, obj in sorted(syscalls.all(), key=lambda x: arch_syscall_number(arch, x)):
        syscall_number = getattr(obj, arch)
        if syscall_number is not None:
            enum_number = syscall_number
        else:
            enum_number = undefined_syscall
            undefined_syscall -= 1
        f.write("  %s = %d,\n" % (name, enum_number))
    f.write("  SYSCALL_COUNT,\n")
    f.write("};\n")
    f.write("\n")

def write_syscall_enum_for_tests(f, arch):
    f.write("enum Syscalls {\n")
    undefined_syscall = -1
    for name, obj in sorted(syscalls.all(), key=lambda x: arch_syscall_number(arch, x)):
        syscall_number = getattr(obj, arch)
        if syscall_number is not None:
            enum_number = syscall_number
        else:
            enum_number = undefined_syscall
            undefined_syscall -= 1
        f.write("  RR_%s = %d,\n" % (name, enum_number))
    f.write("};\n")
    f.write("\n")

def write_syscallname_arch(f):
    f.write("template <typename Arch> static std::string syscallname_arch(int syscall);\n")
    f.write("\n");
    for specializer, arch in [("X86Arch", "x86"), ("X64Arch", "x64"), ("ARM64Arch", "generic")]:
        f.write("template <> std::string syscallname_arch<%s>(int syscall) {\n" % specializer)
        f.write("  switch (syscall) {\n");
        def write_case(name):
            f.write("    case %(specializer)s::%(syscall)s: return \"%(syscall)s\";\n"
                    % { 'specializer': specializer, 'syscall': name })
        for name, _ in syscalls.for_arch(arch):
            write_case(name)
        f.write("    default: {")
        f.write("      char buf[100];")
        f.write("      sprintf(buf, \"<unknown-syscall-%d>\", syscall);")
        f.write("      return buf;\n")
        f.write("    }\n")
        f.write("  }\n")
        f.write("}\n")
        f.write("\n")

def write_syscall_record_cases(f):
    def write_recorder_for_arg(syscall, arg):
        arg_descriptor = getattr(syscall, 'arg' + str(arg), None)
        if isinstance(arg_descriptor, str):
            f.write("    syscall_state.reg_parameter<%s>(%d);\n"
                    % (arg_descriptor, arg))
    for name, obj in syscalls.all():
        # Irregular syscalls will be handled by hand-written code elsewhere.
        if isinstance(obj, syscalls.RegularSyscall):
            f.write("  case Arch::%s:\n" % name)
            for arg in range(1,6):
                write_recorder_for_arg(obj, arg)
            f.write("    return PREVENT_SWITCH;\n")

has_syscall = string.Template("""inline bool
has_${syscall}_syscall(SupportedArch arch) {
  switch (arch) {
    case x86:
      return X86Arch::${syscall} >= 0;
    case x86_64:
      return X64Arch::${syscall} >= 0;
    case aarch64:
      return ARM64Arch::${syscall} >= 0;
    default:
      DEBUG_ASSERT(0 && "unsupported architecture");
      return false;
  }
}
""")

is_syscall = string.Template("""inline bool
is_${syscall}_syscall(int syscallno, SupportedArch arch) {
  switch (arch) {
    case x86:
      return syscallno >= 0 && syscallno == X86Arch::${syscall};
    case x86_64:
      return syscallno >= 0 && syscallno == X64Arch::${syscall};
    case aarch64:
      return syscallno >= 0 && syscallno == ARM64Arch::${syscall};
    default:
      DEBUG_ASSERT(0 && "unsupported architecture");
      return false;
  }
}
""")

syscall_number = string.Template("""inline int
syscall_number_for_${syscall}(SupportedArch arch) {
  switch (arch) {
    case x86:
      DEBUG_ASSERT(X86Arch::${syscall} >= 0);
      return X86Arch::${syscall};
    case x86_64:
      DEBUG_ASSERT(X64Arch::${syscall} >= 0);
      return X64Arch::${syscall};
    case aarch64:
      DEBUG_ASSERT(ARM64Arch::${syscall} >= 0);
      return ARM64Arch::${syscall};
    default:
      DEBUG_ASSERT(0 && "unsupported architecture");
      return -1;
  }
}
""")

def write_syscall_helper_functions(f):
    def write_helpers(syscall):
        subs = { 'syscall': syscall }
        f.write(has_syscall.safe_substitute(subs))
        f.write(is_syscall.safe_substitute(subs))
        f.write(syscall_number.safe_substitute(subs))

    for name, obj in syscalls.all():
        write_helpers(name)

def write_check_syscall_numbers(f):
    for name, obj in syscalls.all():
        # XXX hard-coded to x86 currently
        if not obj.x86:
            continue
        f.write("""static_assert(X86Arch::%s == SYS_%s, "Incorrect syscall number for %s");\n"""
                % (name, name, name))

generators_for = {
    'AssemblyTemplates': lambda f: assembly_templates.generate(f),
    'CheckSyscallNumbers': write_check_syscall_numbers,
    'SyscallEnumsX86': lambda f: write_syscall_enum(f, 'x86'),
    'SyscallEnumsX64': lambda f: write_syscall_enum(f, 'x64'),
    'SyscallEnumsGeneric': lambda f: write_syscall_enum(f, 'generic'),
    'SyscallEnumsForTestsX86': lambda f: write_syscall_enum_for_tests(f, 'x86'),
    'SyscallEnumsForTestsX64': lambda f: write_syscall_enum_for_tests(f, 'x64'),
    'SyscallEnumsForTestsGeneric': lambda f: write_syscall_enum_for_tests(f, 'generic'),
    'SyscallnameArch': write_syscallname_arch,
    'SyscallRecordCase': write_syscall_record_cases,
    'SyscallHelperFunctions': write_syscall_helper_functions,
}

def main(argv):
    filename = argv[0]
    base, extension = os.path.splitext(os.path.basename(filename))

    if os.access(filename, os.F_OK):
        with open(filename, 'r') as f:
            before = f.read()
    else:
        before = ""

    stream = StringIO()
    generators_for[base](stream)
    after = stream.getvalue()
    stream.close()

    if before != after:
        with open(filename, 'w') as f:
            f.write(after)

if __name__ == '__main__':
    main(sys.argv[1:])



from util import *

gdb_signals = [
  '',
  'SIGHUP',
  'SIGINT',
  'SIGQUIT',
  'SIGILL',
  'SIGTRAP',
  'SIGABRT',
  'SIGBUS',
  'SIGFPE',
  '#SIGKILL',
  'SIGUSR1',
  'SIGSEGV',
  'SIGUSR2',
  'SIGPIPE',
  'SIGALRM',
  'SIGTERM',
  '#SIGSTKFLT',
  'SIGCHLD',
  'SIGCONT',
  '#SIGSTOP',
  'SIGTSTP',
  'SIGTTIN',
  'SIGTTOU',
  'SIGURG',
  'SIGXCPU',
  'SIGXFSZ',
  'SIGVTALRM',
  'SIGPROF',
  'SIGWINCH',
  'SIGIO',
  '#SIGPWR',
  'SIGSYS']

for sig in range(32,65):
    gdb_signals.append('SIG%d'%sig)

for sig in range(1,65):
    gdb_sig = gdb_signals[sig]
    if not gdb_sig.startswith('#'):
        send_gdb('handle %s stop'%gdb_sig)
        if gdb_sig == 'SIGINT' or gdb_sig == 'SIGTRAP':
            expect_gdb('used by the debugger')
            send_gdb('y')
        expect_gdb('(rr)')
        send_gdb('c')
        expect_gdb('received signal %s'%gdb_sig)

ok()



from util import *
import re

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1, breakpoint')

send_gdb('fin')
index = expect_list([re.compile(r'signal\(i, handle_sigrt\)'), re.compile('breakpoint')])
if index == 1:
    send_gdb('n')
    expect_gdb(r'signal\(i, handle_sigrt\)')

send_gdb('n')
expect_gdb(r'raise\(i\)')

send_gdb('n')
expect_gdb('Program received signal SIG34')

send_gdb('stepi')
send_gdb('n')
expect_gdb(r'atomic_printf\("Caught signal')

ok()



from util import *

send_gdb('break main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('n');
send_gdb('n');
send_gdb('reverse-next');
send_gdb('c');

expect_gdb('exited normally')

ok()



from util import *

send_gdb('break my_write')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('s')

send_gdb('c')
# First E may be printed by itself during 's'
expect_rr('XIT-SUCCESS')
expect_gdb('exited normally')

ok()



from util import *

send_gdb('b C')
expect_gdb('Breakpoint 1')

def check_breakpoint():
    expect_rr('calling C')

    expect_gdb('Breakpoint 1, C')

    send_gdb('bt')
    expect_gdb('#0[^C]+C[^#]+#1[^B]+B[^#]+#2[^A]+A[^#]+#3[^m]+main')

send_gdb('c')
check_breakpoint()

restart_replay()
check_breakpoint()

restart_replay()
check_breakpoint()

ok()



from util import *
import re

# Test that the elapsed-time GDB command returns a time >= 1.0 at a breakpoint
# after sleep(1);


send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('c')
send_gdb('elapsed-time')

expect_gdb(re.compile(r'(?<=Elapsed Time \(s\): )[0-9.]+'))
elapsed_time = float(last_match().group(0))
sleep_time = 1.0
if elapsed_time < sleep_time:
    failed("ERROR ... The reported elapsed time after sleeping for " +
           "{sleep_time} (s) was {elapsed_time}".format(
               sleep_time=sleep_time, elapsed_time=elapsed_time))

ok()



from util import *

send_gdb('handle SIGKILL stop')

send_gdb('c')
expect_gdb('SIGKILL')

send_gdb('watch -l x')
expect_gdb('Hardware[()/a-z ]+watchpoint 1')

send_gdb('rc')
expect_gdb('watchpoint 1')
expect_gdb('New value = 1564779003')

ok()



from util import *
import re

send_gdb('b my_memmove')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb(re.compile(r'Buf is at ([^ \n]+)'))
buf = eval(last_match().group(1));
expect_gdb('Breakpoint 1')

send_gdb('watch -l *(char*)%d'%(buf + 14000))
expect_gdb('atchpoint 2')

send_gdb('c')
expect_gdb('atchpoint 2')

ok()



import re
from util import *

send_gdb('set scheduler-locking off')

send_gdb('b hit_barrier')
expect_gdb('Breakpoint 1')

send_gdb('b ready')
expect_gdb('Breakpoint 2')

send_gdb('c')
expect_gdb('Breakpoint 2, ready')

bps = set(('A', 'B', 'C'))
for bp in bps:
    send_gdb('b '+ bp +'')
    expect_gdb('Breakpoint \d')

expect_gdb(r'\(rr\)')

hit_bps = { 'A': 0, 'B': 0, 'C': 0 }

events = [ re.compile(r'Breakpoint 1, hit_barrier'),
           re.compile(r'Breakpoint \d, ([ABC])'),
           re.compile(r'Remote connection closed'),
           re.compile(r'internal-error:'),
           re.compile(r'Cannot find bounds of current function'),
           re.compile(r'\(rr\)') ]
next_cmd = 's'
while 1:
    send_gdb(next_cmd)
    next_cmd = 's'
    i = expect_list(events)
    if 0 == i:
        break
    if 2 == i or 3 == i:
        assert False, 'Program stopped unexpectedly, review gdb_rr.log'
    if 4 == i:
        expect_gdb(r'\(rr\)')
        next_cmd = 'stepi'
        continue
    if 5 == i:
        continue

    bp = last_match().group(1)
    assert not hit_bps[bp]
    hit_bps[bp] = 1
    expect_gdb(r'\(rr\)')

for bp in hit_bps.keys():
    assert hit_bps[bp]

arch = get_exe_arch()

# The locations the threads are stopped at depends on the architecture.
stopped_locations = {
    # on i386, we sometimes stop in the middle of nowhere
    'i386': ['(0x[0-9a-f]+ in )?__kernel_vsyscall',
             '(0x[0-9a-f]+ in )?_traced_raw_syscall',
              '0x[0-9a-f]+ in \?\?',
             '(0x[0-9a-f]+ in )?__lll_lock_wait',
             '(0x[0-9a-f]+ in )?pthread_barrier_wait',
             '(0x[0-9a-f]+ in )?futex_wait'],
    'i386:x86-64': ['(0x[0-9a-f]+ in )?__lll_lock_wait',
                    '(0x[0-9a-f]+ in )?pthread_barrier_wait',
                    '(0x[0-9a-f]+ in )?futex_wait',
                    '0x0*70000002 in \?\?'],
    'aarch64': ['(0x[0-9a-f]+ in )?futex_wait'],
}

location_regex = '|'.join(stopped_locations[arch])

send_gdb('info threads')
expect_gdb(r'1\s+Thread.+hit_barrier')
expect_gdb(r'\(rr\)')

send_gdb('info threads')
expect_gdb(r'2\s+Thread.+?(?:%s)' % location_regex)
expect_gdb(r'\(rr\)')

send_gdb('info threads')
expect_gdb(r'3\s+Thread.+?(?:%s)' % location_regex)
expect_gdb(r'\(rr\)')

ok()



from util import *
import re

NUM_THREADS = 10

send_gdb('b hit_barrier')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1, hit_barrier')

arch = get_exe_arch()

# The locations the threads are stopped at depends on the architecture.
stopped_locations = {
    # on i386, we sometimes stop in the middle of nowhere
    'i386': ['(0x[0-9a-f]+ in )?__kernel_vsyscall',
             '(0x[0-9a-f]+ in )?_traced_raw_syscall',
             '0x[0-9a-f]+ in \?\?',
             '(0x[0-9a-f]+ in )?__lll_lock_wait',
             '(0x[0-9a-f]+ in )?pthread_barrier_wait',
             '(0x[0-9a-f]+ in )?futex_wait'],
    'i386:x86-64': ['(0x[0-9a-f]+ in )?__lll_lock_wait',
                    '(0x[0-9a-f]+ in )?pthread_barrier_wait',
                    '(0x[0-9a-f]+ in )?futex_wait',
                    '0x0*70000002 in \?\?'],
    'aarch64': ['(0x[0-9a-f]+ in )?futex_wait']
}

for i in range(NUM_THREADS + 1, 1, -1):
    # The threads are at the kernel syscall entry, or either the
    # traced/untraced entry reached through the rr monkeypatched one.
    # Rarely, non-main threads have been observed to be reordered (i.e. gdb
    # did not number them in order of creation). This does not seem to be a bug
    # so tolerate it.
    send_gdb('info threads')
    expect_gdb(r'%d\s+Thread[^(]+\(BP-THREAD-[0-9]+\)\s+(?:%s)'%
               (i, '|'.join(stopped_locations[arch])))
    expect_gdb(r'\(rr\)')

send_gdb('info threads')
expect_gdb(r'1\s+Thread[^h]+hit_barrier \(\)')

ok()



from util import *
import re

send_gdb('handle SIGKILL stop')
send_gdb('c')
expect_gdb('received signal SIGKILL')

restart_replay()
expect_gdb('received signal SIGKILL')

send_gdb('c')
expect_gdb('exited normally')

ok()



#!/usr/bin/env python3

import io
import os
import sys

# Currently the rr page is not copied into the trace. If you want to change
# the contents of the rr page, think carefully about trace compatibility.
# One option would be to save the (replay) rr page into the trace and have
# replay default to the old rr page if the saved page is not present.

def write_rr_page(f, is_64, is_arm, is_replay):
    # The length of each code sequence must be RR_PAGE_SYSCALL_STUB_SIZE.
    # The end of each syscall instruction must be at offset
    # RR_PAGE_SYSCALL_INSTRUCTION_END.
    if is_arm:
        bytes = bytearray([
            0x1, 0x0, 0x0, 0xd4, # svc #0
            0xc0, 0x03, 0x5f, 0xd6, # ret
        ])
        nocall_bytes = bytearray([
            0x0, 0x0, 0x80, 0xd2, # movz x0, #0
            0xc0, 0x03, 0x5f, 0xd6, # ret
        ])
        trap_bytes = bytearray([
            0x0, 0x0, 0x20, 0xd4, # brk #0
            0xc0, 0x03, 0x5f, 0xd6, # ret
        ])
    else:
        if is_64:
            bytes = bytearray([
                0x0f, 0x05, # syscall
                0xc3, # ret
            ])
        else:
            bytes = bytearray([
                0xcd, 0x80, # int 0x80
                0xc3, # ret
            ])
        nocall_bytes = bytearray([
            0x31, 0xc0, # xor %eax,%eax
            0xc3, # ret
        ])
        trap_bytes = bytearray([
            0x90, # nop
            0xcc, # int3
            0xc3, # ret
        ])

    # traced
    f.write(bytes)
    # privileged traced
    f.write(bytes)

    # untraced
    f.write(bytes)
    # untraced replay-only
    if is_replay:
        f.write(bytes)
    else:
        f.write(nocall_bytes)
    # untraced record-only
    if is_replay:
        f.write(nocall_bytes)
    else:
        f.write(bytes)

    # privileged untraced
    f.write(bytes)
    # privileged untraced replay-only
    if is_replay:
        f.write(bytes)
    else:
        f.write(nocall_bytes)
    # privileged untraced record-only
    if is_replay:
        f.write(nocall_bytes)
    else:
        f.write(bytes)

    # untraced replay assist
    if is_replay:
        f.write(trap_bytes)
    else:
        f.write(bytes)

    ff_bytes = bytearray([0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff])
    f.write(ff_bytes)

generators_for = {
    'rr_page_32': lambda stream: write_rr_page(stream, False, False, False),
    'rr_page_64': lambda stream: write_rr_page(stream, True, False, False),
    'rr_page_arm64': lambda stream: write_rr_page(stream, True, True, False),
    'rr_page_32_replay': lambda stream: write_rr_page(stream, False, False, True),
    'rr_page_64_replay': lambda stream: write_rr_page(stream, True, False, True),
    'rr_page_arm64_replay': lambda stream: write_rr_page(stream, True, True, True),
}

def main(argv):
    filename = argv[0]
    base = os.path.basename(filename)

    if os.access(filename, os.F_OK):
        with open(filename, 'rb') as f:
            before = f.read()
    else:
        before = ""

    stream = io.BytesIO()
    generators_for[base](stream)
    after = stream.getvalue()
    stream.close()

    if before != after:
        with open(filename, 'wb') as f:
            f.write(after)

if __name__ == '__main__':
    main(sys.argv[1:])



from util import *

send_gdb('break main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('watch -l var')
expect_gdb('Hardware[()/a-z ]+watchpoint 2')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 42')

send_gdb('p atomic_printf("hello%s", "kitty")')
expect_gdb('hellokitty')

send_gdb('delete 2')

send_gdb('break pthread_join')
expect_gdb('Breakpoint 3')
send_gdb('c')
expect_gdb('Breakpoint 3')

send_gdb('p atomic_printf("hello%s", "kitty")')
expect_gdb('hellokitty')

send_gdb('c')
expect_gdb('EXIT-SUCCESS')

ok()



from util import *
import re

# Restart at the first debuggable event, which will be in a different
# process! We should stay focused on the child process, instead of
# trying to switch to that process. At least we shouldn't crash.
restart_replay(1)
expect_gdb('(rr)')
expect_list([re.compile('Program stopped'), re.compile('exited normally')])

ok()



import collections
import sys
import re
from util import *

arch = get_exe_arch()

ArchInfo = collections.namedtuple('ArchInfo', ['ip_name'])
regex_info = {
    'i386': ArchInfo('eip'),
    'i386:x86-64': ArchInfo('rip'),
    'aarch64': ArchInfo('pc'),
}

syscall_re = re.compile("`SYSCALL: <unknown-syscall--1>' \\(state:EXITING_SYSCALL\\)")
sched_re = re.compile("`SCHED'")
eip_re = re.compile("%s:(0x[a-f0-9]+)" % regex_info[arch].ip_name)

sched_enabled = False
eip_enabled = False
eip = None
while True:
    line = sys.stdin.readline()
    if not line:
        break
    if syscall_re.search(line):
        sched_enabled = True
    if sched_enabled and sched_re.search(line):
        eip_enabled = True
    if eip_enabled:
        m = eip_re.search(line)
        if m:
            eip = m.group(1)
            break

if eip is None:
    failed('%s not found' % regex_info[arch].ip_name)

# The SCHED after getgid may land in libc, which might not be loaded yet, in
# which case setting a breakpoint there will cause gdb to barf. So run to
# 'main' at which point libc is definitely loaded.
send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('b *%s'%eip)
expect_gdb('Breakpoint 2')
send_gdb('c')
expect_gdb('Breakpoint 2')
expect_gdb('(rr)')

if arch == 'aarch64':
    send_gdb('p/x *(uint32_t*)$pc')
    expect_gdb('0x([a-f0-9]+)')
    if last_match().group(1) == 'd4200000':
        failed('saw breakpoint at current instruction')
elif arch == 'i386' or arch == 'i386:x86-64':
    send_gdb('p/x *(char*)$pc')
    expect_gdb('0x([a-f0-9]+)')
    if last_match().group(1) == 'cc':
        failed('saw breakpoint at current instruction')
else:
    failed('Add check for this architecture')

ok()



from util import *
import re

send_gdb('hbreak *syscall_instruction')
expect_gdb('breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('stepi')
send_gdb('print $pc == syscall_instruction + 2')
expect_gdb('= 1')
send_gdb('reverse-stepi')
send_gdb('print $pc == syscall_instruction')
expect_gdb('= 1')

ok()



from util import *

# Test conditional hardware breakpoint. For some reason
# gdb doesn't seem to use agent conditions for hardware
# watchpoint types other than conditional breakpoints.

send_gdb('hbreak main')
expect_gdb('reakpoint 1')
send_gdb('cond 1 var==0x1234')
send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited normally')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('checkpoint')
send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited normally')

ok()



from util import *
import re

send_gdb('b atomic_puts')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

for i in range(1,100):
    send_gdb('checkpoint')
    expect_gdb('Checkpoint %d '%i)
    send_gdb('stepi')
    send_gdb('restart %d'%i)
    expect_gdb('stopped')
    send_gdb('delete checkpoint %d'%i)
    send_gdb('stepi')
    send_gdb('stepi')

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1, main')

send_gdb('p *0xf')
expect_gdb('Cannot access memory at address 0xf')

send_gdb('p *0xffffffff')
expect_gdb('Cannot access memory at address 0xffffffff')

ok()



from util import *

send_gdb('break main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('break traced_raw_syscall')
expect_gdb('Breakpoint 2')
send_gdb('c')
expect_gdb('Breakpoint 2')

send_gdb('where')
expect_gdb(' main ')

ok()



from util import *

send_gdb('b breakpointA')
expect_gdb('Breakpoint 1')
send_gdb('c')

expect_gdb('i=0')
send_gdb('c')
expect_gdb('i=1')
send_gdb('c')
expect_gdb('i=2')
send_gdb('c')
expect_gdb('i=3')
send_gdb('c')
expect_gdb('i=4')

send_gdb('forward')
expect_gdb("Can't go forward. No more history entries.")

send_gdb('back')
expect_gdb('i=3')
send_gdb('back')
expect_gdb('i=2')
send_gdb('back')
expect_gdb('i=1')

# A diversion should not interfere with the history
send_gdb('call strlen("abcd")')

send_gdb('back')
expect_gdb('i=0')

# Clear the forward stack by pushing a new entry
send_gdb('c')
expect_gdb('i=1')
send_gdb('forward')
expect_gdb("Can't go forward. No more history entries.")

send_gdb('back')
expect_gdb('i=0')

send_gdb('d 1')
expect_gdb('c')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('up')
send_gdb('b *space')
expect_gdb('Breakpoint 2')

# Should hit breakpoint 2 here rather than making it all the way back to
# the first instance of breakpoint 1.
send_gdb('rc')
expect_gdb('Breakpoint 2')

ok()



from util import *

send_gdb('handle SIGKILL stop')

send_gdb('c')
expect_gdb('SIGSEGV')
send_gdb('stepi')
expect_gdb('SIGSEGV')
send_gdb('stepi')
expect_gdb('SIGKILL')

send_gdb('reverse-stepi')
expect_gdb('SIGSEGV')
send_gdb('reverse-stepi')
expect_gdb('SIGSEGV')
send_gdb('reverse-continue')
expect_gdb('stopped')

ok()



from util import *
import re

send_gdb('b hit_barrier')
expect_gdb('Breakpoint 1')

send_gdb('b joined_threads')
expect_gdb('Breakpoint 2')

send_gdb('c')
expect_gdb('Breakpoint 1, hit_barrier')

send_gdb('info thr')
expect_gdb('2    Thread')

send_gdb('thr 2')
expect_gdb('Switching to thread 2')

send_gdb('c')
# TODO: with the gdb in fedora 19, if a thread dies while it's the
# resume target, then rr notifies gdb, but gdb doesn't ask for a new
# thread list.  This seems like a gdb bug, because we don't have any
# other way to notify gdb of thread death, and the same code works
# just fine in concurrent ubuntu and older versions.
#
# So we work around that problem by returning this special error code
# to the user.  Once gdb has made this mistake, the debugging session
# is "stuck" because won't let any other threads continue.  But at
# least this error code tells the user that they need to restart the
# session.
expect_gdb(re.compile(
    r'Breakpoint 2, joined_threads|Remote failure reply: E10'))

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('b atomic_puts')
expect_gdb('Breakpoint 2')
send_gdb('c')
expect_gdb('Breakpoint 2')

send_gdb('checkpoint')
expect_gdb('Checkpoint 1 at')
send_gdb('restart 8')
send_gdb('restart -1')
send_gdb('restart abc')
send_gdb('restart 1')
expect_gdb('stopped')
send_gdb('c')
# If rr crashes, a 'restart' will re-run the program directly under gdb from
# the beginning. If that happens, we'll stop at breakpoint 1, not exit normally.
expect_gdb('xited normally')

ok()



from util import *
import re

send_gdb('handle SIGKILL stop')

send_gdb('b main')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('b __before_poll_syscall_breakpoint')
expect_gdb('Breakpoint 2')
send_gdb('c')
index = expect_list([re.compile(r'Breakpoint 2'), re.compile(r'SIGKILL')])
if index == 0:
  # This is testing that we can reverse-step without crashing rr.
  send_gdb('reverse-stepi')
  send_gdb('c')
  expect_gdb('Breakpoint 2')

ok()



from util import *
import re

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('p (int)open("diversion_print.out",0102,0600)')
expect_gdb('\$1 = 3')
# This FD is confused with that during record. That's ok, just make sure
# operations on this during the diversion actually go to our file.

send_gdb('p (int)write(3, "DIVERSION-SUCCESS", 17)')
expect_gdb('\$2 = 17')

ok()



import re
from util import *

send_gdb('b print_dot if checker()')
expect_gdb('Breakpoint 1')
send_gdb('watch dot_counter if checker()')
expect_gdb('Hardware watchpoint 2')
send_gdb('c')
expect_rr('EXIT-SUCCESS')

ok()



from util import *

restart_replay()
# A single EXIT-SUCCESS is expected since the child process to which we have
# attached only prints one, and it exits before the parent prints its
# EXIT-SUCCESS.
expect_rr('EXIT-SUCCESS')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('watch -l *p')
expect_gdb('Hardware[()/a-z ]+watchpoint 2')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 97')

send_gdb('c')
expect_gdb('Old value = 97')
expect_gdb('New value = 0')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 98')

send_gdb('c')
expect_gdb('Old value = 98')
expect_gdb('New value = 0')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = <unreadable>')

send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited normally')

ok()



from util import *

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1, breakpoint')

send_gdb('f 1')
expect_gdb('(rr)')

send_gdb('p big')
expect_gdb("bytes = 'Z'")

ok()



from util import *

send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited')

send_gdb('b main')
expect_gdb('Breakpoint 1')

restart_replay()
expect_gdb('Breakpoint 1')
send_gdb('checkpoint')
expect_gdb('Checkpoint 1 at')
send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited')

restart_replay()
expect_gdb('Breakpoint 1')
send_gdb('checkpoint')
expect_gdb('Checkpoint 2 at')
send_gdb('c')
expect_rr('EXIT-SUCCESS')
expect_gdb('exited')

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('n')
send_gdb('break')
expect_gdb('Breakpoint 2')

send_gdb('reverse-next')
expect_gdb('Breakpoint 1')

send_gdb('next')
expect_gdb('Breakpoint 2')

ok()



from util import *

send_gdb('break break_function')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('check')
expect_gdb('Checkpoint 1')
send_gdb('next')
send_gdb('restart 1')
expect_gdb('break_function')

send_gdb('continue')
expect_rr('EXIT-SUCCESS')

ok()



#!/usr/bin/env python3

import errno
import glob
import os
import re
import shutil
import subprocess
import sys
import tempfile
from urllib.request import urlretrieve
from urllib.error import HTTPError, ContentTooShortError

# Usage: rr-collect-symbols.py <trace-dir> [<url> | <path>]
#
# Given a <url>, downloads the zip/.tar.zst file at <url>, uncompresses it,
# runs "gunzip" on any .gz files, and for any ELF files found whose build-ids
# match the build-id of an ELF file in the trace, moves it into the trace.
#
# Given a <path>, which must contain a .build-id directory with the usual
# structure (e.g. as Ubuntu and Fedora create under /usr/lib/debug), searches
# the directory tree for any ELF files whose build-ids match the build-id of
# an ELF file in the trace and copies them into the trace. <path> defaults to
# "/usr/lib/debug", which will grab any available system debuginfo files
# in Ubuntu and Fedora at least.
#
# This script assumes that the trace-dir has been packed via `rr pack` so all
# relevant files actually appear in the trace-dir.
# It also assumes rr is on the PATH.
#
# The debuginfo files are placed in the trace under a "debug" subdirectory,
# in a ".build-id" subdirectory with the usual structure.
#
# If a debuginfo file contains a .gnu_debugaltlink section then we also
# attempt to find the referenced file and copy it into the trace with the
# same file name as the .debug file, but with a .sup suffix.

if len(sys.argv) < 2:
    print("Usage: rr-collect-symbols.py <trace-dir> [<url> | <path>]", file=sys.stderr)
    sys.exit(1)
trace_dir = sys.argv[1]

if len(sys.argv) < 3:
    source = "/usr/lib/debug"
else:
    source = sys.argv[2]

rr_buildid = subprocess.Popen(["rr", "buildid"],
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE)

def build_id_for(file):
    global rr_buildid
    rr_buildid.stdin.write(("%s\n"%file).encode('utf-8'))
    try:
        rr_buildid.stdin.flush()
    except BrokenPipeError:
        print("Can't write to rr, termination code %s"%rr_buildid.returncode, file=sys.stderr)
        sys.exit(2)
    return rr_buildid.stdout.readline().rstrip().decode('utf-8')

altref_regex = re.compile(rb"^\s+\[\s*0\]\s+(.*)");

def find_altref(file):
    proc = subprocess.Popen(["readelf", "-p", ".gnu_debugaltlink", file], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    try:
        for line in proc.stdout:
            m = altref_regex.match(line)
            if m:
                return m.group(1).rstrip()
    finally:
        proc.wait()
    return None

def find_altref_for_trace_file(trace_file, altref):
    proc = subprocess.Popen(["rr", "filename", trace_file], stdout=subprocess.PIPE)
    try:
        for line in proc.stdout:
            file = line.rstrip()
            altref_file = os.path.join(os.path.dirname(file), altref)
            if os.path.isfile(altref_file):
                return altref_file
    finally:
        proc.wait()
    return None

def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

def collect_trace_build_ids():
    ret = {}
    for file in glob.iglob("%s/mmap_*"%trace_dir):
        build_id = build_id_for(file)
        if build_id:
            ret[build_id] = True
            altref = find_altref(file)
            if altref:
                altref_file = find_altref_for_trace_file(file, altref)
                if not altref_file:
                    raise "Can't find alt file %s for %s"%(altref, file)
                dir = "%s/debug/.build-id/%s"%(trace_dir, build_id[:2])
                mkdir_p(dir)
                dst = "%s/%s.sup"%(dir, build_id[2:])
                subprocess.check_call(["cp", "--preserve", "-f", "--reflink=auto", altref_file, dst])
    return ret

trace_build_ids = collect_trace_build_ids()

def collect_archive(url):
    is_tar_zst = url.endswith(".tar.zst")
    tmp_dir = tempfile.mkdtemp(dir=trace_dir)
    if is_tar_zst:
        tmp_file_name = "%s/archive.tar.zst"%tmp_dir
    else:
        # Assume its a ZIP
        tmp_file_name = "%s/archive.zip"%tmp_dir
    try:
        (file, headers) = urlretrieve(url, tmp_file_name)
    except (HTTPError, ContentTooShortError) as exc:
        print("Failed to load archive %s: %s"%(url, exc), file=sys.stderr)
        sys.exit(2)
    if is_tar_zst:
        subprocess.check_call(["tar", "-C", tmp_dir, "-I", "zstd", "-xvf", file])
    else:
        subprocess.check_call(["unzip", "-d", tmp_dir, file])
    os.remove(file)

    for root, dirs, files in os.walk(tmp_dir):
        for name in files:
            file = os.path.join(root, name)
            if file.endswith(".gz"):
                subprocess.check_call(["gunzip", file])
                file = file[:-3]
            build_id = build_id_for(file)
            if build_id and build_id in trace_build_ids:
                dir = "%s/debug/.build-id/%s"%(trace_dir, build_id[:2])
                mkdir_p(dir)
                dst = "%s/%s.debug"%(dir, build_id[2:])
                os.rename(file, dst)
            else:
                os.remove(file)

    shutil.rmtree(tmp_dir)

def collect_filesystem(path):
    for root, dirs, files in os.walk(path):
        for name in files:
            file = os.path.join(root, name)
            if not os.path.islink(file):
                build_id = build_id_for(file)
                if build_id and build_id in trace_build_ids:
                    dir = "%s/debug/.build-id/%s"%(trace_dir, build_id[:2])
                    mkdir_p(dir)
                    dst = "%s/%s.debug"%(dir, build_id[2:])
                    subprocess.check_call(["cp", "--preserve", "-f", "--reflink=auto", file, dst])
                    altref = find_altref(file)
                    if altref:
                        altref_file = os.path.join(os.path.dirname(file), altref.decode('utf-8'))
                        dst = "%s/%s.sup"%(dir, build_id[2:])
                        subprocess.check_call(["cp", "--preserve", "-f", "--reflink=auto", altref_file, dst])

if re.search("^[^:/]+:", source):
    collect_archive(source)
else:
    collect_filesystem(source)

rr_buildid.terminate()



from util import *
import re

send_gdb('c')
index = expect_list([re.compile(r'exited normally'), re.compile(r'Program received signal SIGUSR1')])

if index == 1:
    send_gdb('c')
    expect_gdb('exited normally')

ok()



from util import *

send_gdb('break breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('finish')

send_gdb('watch -l buf[100]')
expect_gdb('Hardware[()/a-z ]+watchpoint 2')

send_gdb('c')
expect_gdb('Old value = 0')
expect_gdb('New value = 100')

ok()



from util import *

send_gdb('break atomic_puts')
expect_gdb('Breakpoint 1')
send_gdb('watch var')
expect_gdb('Hardware watchpoint 2')

send_gdb('c')
expect_gdb('Hardware watchpoint 2')
expect_gdb('Old value = 0')
expect_gdb('New value = 42')

send_gdb('c')
expect_gdb('Hardware watchpoint 2')
expect_gdb('Old value = 42')
expect_gdb('New value = 1337')

send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('reverse-cont')
expect_gdb('Hardware watchpoint 2')
expect_gdb('Old value = 1337')
expect_gdb('New value = 42')

send_gdb('reverse-cont')
expect_gdb('Hardware watchpoint 2')
expect_gdb('Old value = 42')
expect_gdb('New value = 0')

send_gdb('reverse-cont')
expect_gdb('stopped')

ok()



from util import *

send_gdb('b breakpoint')
expect_gdb('Breakpoint 1')
send_gdb('c')
expect_gdb('Breakpoint 1')

send_gdb('set scheduler-locking off')
send_gdb('reverse-step')

send_gdb('c')
expect_gdb('Breakpoint 1')

ok()



from util import *

send_gdb('b main')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1, main')

send_gdb('call (int)exit(0)')
expect_gdb('while in a function called from GDB')

restart_replay()
expect_gdb('Breakpoint 1, main')

ok()



from util import *

send_gdb('break breakpoint_fn')
expect_gdb('Breakpoint 1')

send_gdb('c')
expect_gdb('Breakpoint 1')
send_gdb('print tlsvar')
expect_gdb(' = 97')

send_gdb('reverse-stepi')
send_gdb('print tlsvar')
expect_gdb(' = 97')

ok()
